import {
  ProtocolRequest,
  ProtocolResponse,
  ProtocolEncoder,
  PacketBody,
  PacketBodyValue,
  ProtocolDecoder,
  ProtocolMessage,
  BodyStringDivider,
  ProtocolStream
} from '@ucap-webmessenger/protocol';
import { FileType } from '../types/file.type';

export interface InfoRequest extends ProtocolRequest {
  // 대화방SEQ(s)
  roomSeq: string;
  // { 파일타입 } cf) I : 이미지 V: 동영상 F: 파일 "" 빈값이면 모든 타입을 내려줌
  type: FileType;
}

export interface FileInfo {
  // 대화방SEQ
  roomSeq: string;
  // 이벤트SEQ
  eventSeq: number;
  // 파일SEQ
  seq: number;
  // 송신자SEQ
  senderSeq: number;
  // 파일타입
  type: FileType;
  // 파일이름
  name: string;
  // 파일URL
  url: string;
  // 파일크기(byte)
  size: number;
  // 전송일시
  sendDate: string;
  // 수신완료자수
  receivedUserCount: number;
  // 수신자수
  receiverCount: number;
}

export interface InfoData extends ProtocolStream {
  // { 대화방파일정보 }...
  fileInfos: FileInfo[];
}

export interface FileDownloadInfo {
  // 대화방SEQ
  roomSeq: string;
  // 파일SEQ
  seq: number;
  // 사용자SEQ
  userSeq: number;
  // 사용자명
  userName: string;
  // 다운로드일시
  downloadDate: string;
  // DownYN
  isDownload: boolean;
}

export interface InfoCheckData extends ProtocolStream {
  // { 파일다운로드정보 }...
  fileDownloadInfos: FileDownloadInfo[];
}

export interface InfoResponse extends ProtocolResponse {
  // 대화방SEQ(s)
  roomSeq: string;
  // { 파일타입 } cf) I : 이미지 V: 동영상 F: 파일 "" 빈값이면 모든 타입을 내려줌
  type: FileType;
}

export const encodeInfo: ProtocolEncoder<InfoRequest> = (req: InfoRequest) => {
  const bodyList: PacketBody[] = [];

  bodyList.push(
    { type: PacketBodyValue.String, value: req.roomSeq },
    { type: PacketBodyValue.String, value: req.type }
  );
  return bodyList;
};

export const decodeInfoData: ProtocolDecoder<InfoData> = (
  message: ProtocolMessage
) => {
  const fileInfos: FileInfo[] = [];
  message.bodyList.forEach(fileInfo => {
    const info = fileInfo.split(BodyStringDivider);
    let i = 0;
    fileInfos.push({
      roomSeq: info[i],
      eventSeq: Number(info[i++]),
      seq: Number(info[i++]),
      senderSeq: Number(info[i++]),
      type: info[i++] as FileType,
      name: info[i++],
      url: info[i++],
      size: info[i++],
      sendDate: info[i++],
      receivedUserCount: Number(info[i++]),
      receiverCount: Number(info[i++])
    });
  });

  return {
    fileInfos
  } as InfoData;
};

export const decodeInfoCheckData: ProtocolDecoder<InfoCheckData> = (
  message: ProtocolMessage
) => {
  const fileDownloadInfos: FileDownloadInfo[] = [];
  message.bodyList.forEach(fileInfo => {
    const info = fileInfo.split(BodyStringDivider);
    let i = 0;
    fileDownloadInfos.push({
      roomSeq: info[i],
      seq: Number(info[i++]),
      userSeq: Number(info[i++]),
      userName: info[i++],
      downloadDate: info[i++],
      isDownload: info[i++] === 'Y' ? true : false
    });
  });

  return {
    fileDownloadInfos
  } as InfoCheckData;
};

export const decodeInfo: ProtocolDecoder<InfoResponse> = (
  message: ProtocolMessage
) => {
  return {
    roomSeq: message.bodyList[0],
    type: message.bodyList[1] as FileType
  } as InfoResponse;
};
